@@ -6,6 +6,7 @@ pub mod argsort;
 pub mod array_compression;
 pub mod binary_search;
 pub mod bitwise;
+pub mod carmichael_number;
 pub mod chinese_remainder_theorem;
 pub mod cmp;
 pub mod combinatorics;
@@ -33,6 +34,7 @@ pub mod modular_runtime_static;
 pub mod modular_static;
 pub mod power;
 pub mod primality;
+pub mod primality_test_fermat;
 pub mod primality_test_miller_rabin;
 pub mod prime_number;
 pub mod segment_tree;
